/*
    8-6-06
    Copyright Spark Fun Electronics 2007
    Nathan Seidle
    nathan at sparkfun.com
    
    USB Weather board using the SCP1000-D01 SPI pressure sensor 
    and the SHT15 I2C based humidity sensor all running through
    an ATmega8 with a USB interface over an FT232RL VCP connection.

	avrdude -p atmega8 -P lpt1 -c stk200 -U flash:w:USBWeather-v1.hex lfuse:w:0xff:m -U hfuse:w:0xc9:m
	avrdude -p atmega8 -P lpt1 -c stk200 -U flash:w:USBWeather-v14.hex lfuse:w:0xff:m -U hfuse:w:0xc9:m

	avrdude -p atmega8 -P lpt1 -c stk200 -U lfuse:w:0xff:m -U hfuse:w:0xc9:m

*/
#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#define FOSC 10000000 //10MHz external osc

#define BAUD 9600
#define MYUBRR (((((FOSC * 10) / (16L * BAUD)) + 5) / 10) - 1)

#define sbi(var, mask)   ((var) |= (uint8_t)(1 << mask))
#define cbi(var, mask)   ((var) &= (uint8_t)~(1 << mask))

#define STATUS_LED 3

#define DRDY    0
#define CSB     1
#define SCP_MISO    2 //changed
#define SCP_MOSI    3 //changed

#define MOSI    3
#define MISO    4
#define SCK     5

#define SHT_SCK 5
#define SHT_SDA 4

#define CHECK_TEMP 0x03
#define CHECK_HUMD 0x05
#define CHECK_STAT 0x07
#define WRITE_STAT 0x06

#define VERBOSE	1
#define MINIMAL	0

#define TEMP_OFFSET_LOC		0
#define OUTPUT_TYPE_LOC		1
#define REPORT_BATT_LOC		2
#define REPORT_LIGHT_LOC	3

#define BATT_CHANNEL	6
#define LIGHT_CHANNEL	7

#define REPORT 1
#define NO_REPORT 0

//1 = output, 0 = input
//DDRC = 0b11111110; //PC0 = DRDY, PC4 = SDA, PC5 = SCL
#define WRITE_sda() DDRC = DDRC | 0b00010000 //SDA must be output when writing
#define READ_sda()  DDRC = DDRC & 0b11101111 //SDA must be input when reading - don't forget the resistor on SDA!!

void ioinit (void);      // initializes IO
static int uart_putchar(char c, FILE *stream);
uint8_t uart_getchar(void);
static FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);

void EEPROM_write(unsigned int uiAddress, unsigned char ucData);
unsigned char EEPROM_read(unsigned int uiAddress);

uint16_t check_adc(uint8_t channel);

void scp_read(void);
uint8_t SCP_read_register(uint8_t register_name);
uint16_t SCP_read_register16(uint8_t register_name);
void SCP_write_register(uint8_t register_name, uint8_t register_value);
uint8_t SCP_spi_comm(uint8_t outgoing_byte);
void clock_byte(volatile uint16_t my_byte, volatile uint8_t rowNum);

uint8_t sht15_read_byte(void);
void sht15_start(void);
void sht15_send_byte(uint8_t sht15_command);
void sht15_read(void);

void print_decimal(int32_t number, uint8_t place, uint8_t pad);
void delay_ms(uint16_t x); // general purpose delay

void config_menu(void);

int32_t temperature;
int32_t humidity;
int32_t scp_temperature;
int32_t scp_temperature_inF;
uint32_t pressure;
uint16_t counter;

uint16_t global_clock;
volatile uint8_t flag;

volatile uint8_t output_type;
volatile int8_t temp_offset;
volatile uint8_t report_light;
volatile uint8_t report_batt;
volatile uint16_t light_level;
volatile uint16_t batt_level;

ISR (TIMER2_OVF_vect) 
{
    global_clock++;

//    if(global_clock == 977) //977 * .0001024 = 1.000448s
    if(global_clock == 40) //(40 * (250 * .0001024) = 1.024s
    {
        flag = 1; //One second has hit
        global_clock = 0;
    }
    
    //TCNT2 = 246; //256 - 10 = 246 - Preload timer 2 for 10 clicks. Should be 102.4us per click
    TCNT2 = 6; //256 - 250 = 6 - Preload timer 2 for 250 clicks. Should be 102.4us per click
}

int main (void)
{
    uint8_t choice;
	counter = 0;
    
    ioinit (); //Boot up defaults

    while(1)
    {
        sht15_read(); //Run Humidity tests
        scp_read(); //Run Pressure tests
        
		light_level = check_adc(LIGHT_CHANNEL); //Check light sensor
		batt_level = check_adc(BATT_CHANNEL); //Check batt level

		scp_temperature_inF = scp_temperature * 90;
		scp_temperature_inF /= 5;
		scp_temperature_inF += 3200;
		scp_temperature_inF += ((int32_t)temp_offset * 100); //scp_temp_inF is 8222 for 82.22F
		
		scp_temperature = scp_temperature_inF - 3200;
		scp_temperature *= 5;
		scp_temperature /= 90;

		temperature += ((int32_t)temp_offset * 100); //temperature is 8233 for 82.33F
		
		if(output_type == VERBOSE)
		{
            printf("\n\nHumidity=");
            print_decimal(humidity, 2, 4);
            printf(" %%");
    
            printf("\nSHT Temperature=");
            print_decimal(temperature, 2, 5);
            printf(" F");
            
            printf("\nSCP Temperature=");
            print_decimal(scp_temperature, 1, 4);
            printf(" C");
    
            printf("\nSCP Temperature=");
            print_decimal(scp_temperature_inF, 2, 5);
            printf(" F");
    
            printf("\nPressure=");
            print_decimal(pressure, 0, 6);
            printf(" Pa");

            if(report_light == REPORT) printf("\nLight=%d", light_level);
            if(report_batt == REPORT) printf("\nBatt=%d", batt_level);
    
            counter++;
            printf("\nCount=");
            print_decimal(counter, 0, 6);
		}
		else
		{            
			printf("#");
            print_decimal(humidity, 2, 4);
            printf(",");
            print_decimal(temperature, 2, 5);
            printf(",");
            print_decimal(scp_temperature, 1, 4);
            printf(",");
            print_decimal(scp_temperature_inF, 2, 5);
            printf(",");
            print_decimal(pressure, 0, 6);
            printf(",");

            if(report_light == REPORT) printf("%d,", light_level);
            if(report_batt == REPORT) printf("%d,", batt_level);

            counter++;
            print_decimal(counter, 0, 6);        
            printf("$\n");
		}

        sbi(PORTD, STATUS_LED);
        
        while(flag == 0)
		{
			if(UCSR0A & (1<<RXC0))
			{
				choice = UDR0;
				if(choice == 19) config_menu();
				//printf("choice = %d", choice);
			}
		}
        flag = 0;
        
        cbi(PORTD, STATUS_LED);            
    }

    return (0);
}

//Takes user input and records settings
//Temp offset, output type
void config_menu(void)
{
	uint8_t choice;
	
	while(1)
	{
		printf("\n\n\rUSB Weather Board v2.0\n\r");
		printf("======================\n\r");
		printf("1) Temperature offset: %dF\n\r", temp_offset);
		printf("2) Output type: ");
		if (output_type == VERBOSE) printf("VERBOSE");
		else printf("MINIMAL");

		printf("\n3) Light level: ");
		if (report_light == REPORT) printf("Reported");
		else printf("Not reported");

		printf("\n4) Battery level: ");
		if (report_batt == REPORT) printf("Reported");
		else printf("Not reported");
		
		printf("\nx) Exit");
		printf("\n\n\rCommand: ");
		
		choice = uart_getchar();
		
		if(choice == '1')
		{
			printf("Press +/- or 'x' to exit\n\r");
			
			while(1)
			{
				printf("\roffset = %dF  ", temp_offset);
				
				choice = uart_getchar();
				if (choice == 'x') break;
				
				if(choice == '+') temp_offset++;
				if(choice == '-') temp_offset--;
				
				if(temp_offset > 50) temp_offset = 50;
				if(temp_offset < -50) temp_offset = -50;
			}

			choice = 0; //Reset 'x'
		}

		if(choice == '2' && output_type == VERBOSE) 
			output_type = MINIMAL;
		else if(choice == '2' && output_type == MINIMAL) 
			output_type = VERBOSE;

		if(choice == '3' && report_light == REPORT) 
			report_light = NO_REPORT;
		else if(choice == '3' && report_light == NO_REPORT) 
			report_light = REPORT;

		if(choice == '4' && report_batt == REPORT) 
			report_batt = NO_REPORT;
		else if(choice == '4' && report_batt == NO_REPORT) 
			report_batt = REPORT;
			
		if(choice == 'x') break;
	}
		
	EEPROM_write(TEMP_OFFSET_LOC, temp_offset);
	EEPROM_write(OUTPUT_TYPE_LOC, output_type);
	EEPROM_write(REPORT_LIGHT_LOC, report_light);
	EEPROM_write(REPORT_BATT_LOC, report_batt);
}

void ioinit (void)
{
    //1 = output, 0 = input
    DDRB = 0b11101111; //PB4 = MISO 
    DDRC = 0b11111010; //PC0 = DRDY, PC2 = SCP_MISO, PC4 = SDA, PC5 = SCL
    DDRD = 0b11011110; //PORTD (RX on PD0) PD5 = MISO

    // USART Baud rate: 9600
    UBRR0H = (MYUBRR) >> 8;
    UBRR0L = MYUBRR;
    UCSR0B = (1<<RXEN0)|(1<<TXEN0);
    UCSR0C = (3<<UCSZ00);
    
    stdout = &mystdout; //Required for printf init
    
    //Init timer 0 for delay_us timing
	//1,000,000 / 1 = 1,000,000
    TCCR0B = (1<<CS00); //Set Prescaler to 1. CS00=1

    sbi(PORTC, CSB); //Deselect SCP1000
    
    //Init timer 2 for 1 second interrupt timing
    //ASSR = 0;
    TCCR2B = (1<<CS22)|(1<<CS21)|(1<<CS20); //Set Prescaler to 1024. CS22=1, CS21=1,CS20=1
    TIMSK2 = (1<<TOIE2); //Enable Timer 2 Interrupt
    
	//Setup ADC
	ADMUX = (1<<REFS0); //Use AVCC with cap at AREF pin.
	
	/*while(1)
	{
		sbi(PORTD, STATUS_LED);
		delay_ms(1000);
		
		cbi(PORTD, STATUS_LED);
		delay_ms(1000);
	}*/
	
	//Load settings from EEPROM
	output_type = EEPROM_read(OUTPUT_TYPE_LOC);
	temp_offset = EEPROM_read(TEMP_OFFSET_LOC);
	report_light = EEPROM_read(REPORT_LIGHT_LOC);
	report_batt = EEPROM_read(REPORT_BATT_LOC);
	
	if (output_type > 1) output_type = MINIMAL;
	if (temp_offset > 50 || temp_offset < -50)  temp_offset = 0;
	if (report_light > 1) report_light = REPORT;
	if (report_batt > 1) report_batt = REPORT;

	int8_t temp, x;
    
    printf("\n\n Weather USB Firmware v2.0\n");

    for(x = 0 ; x < 5 ; x++)
    {
        sbi(PORTD, STATUS_LED);
        delay_ms(100);

        cbi(PORTD, STATUS_LED);
        delay_ms(100);
    }

	//delay_ms(60); //SCP Wakeup

	//Poll SCP until it is ready
	while(1)
	{
		sbi(PORTC, CSB); //De-Select SPI Device
		delay_ms(10);
		temp = SCP_read_register(0x00);
		temp = SCP_read_register(0x00);
	
		temp = SCP_read_register(0x07); //Check STATUS for LSB = 0
		if ( !(temp & 0x01) ) break;

		#ifdef DEBUG
			printf("\n\nPolling SCP1000... temp = %x", temp);
		#endif
	}

	while(1)
	{
		sbi(PORTC, CSB); //De-Select SPI Device
		delay_ms(10);
		temp = SCP_read_register(0x00);
		temp = SCP_read_register(0x00);

		temp = SCP_read_register(0x1F); //Check DATARD8 for LSB = 1
		if (temp & 0x01) break;

		#ifdef DEBUG
			printf("\n\nWaiting for SCP1000... temp = %x", temp);
		#endif
	}

    //Configure SCP1000 with low noise configuration
    //=====================================
    //write_register(place, thing)
	SCP_write_register(0x02, 0x2D); //Write indirect register address to 0x02
    SCP_write_register(0x01, 0x03); //Write 0x03 to indirect register data
    SCP_write_register(0x03, 0x02); //Write this data to the indirect register
    //delay_ms(100);
	
    //=====================================	

    SCP_write_register(0x03, 0x00); //Reset operation mode
    delay_ms(10);

    SCP_write_register(0x03, 0x0A); //Select High Resolution Mode
    delay_ms(100);

	#ifdef DEBUG
		printf("\nSCP1000 Config complete\n");
	#endif

    sei(); //Enable interrupts
}

//Takes an int and prints a decimal in a given place
//(87004, 3, 0) = 87.004
//10 digit max = 1234567890
//pad will put zeros in front to make the length constant
//(87004, 3, 7) = 0087.004
void print_decimal(int32_t number, uint8_t place, uint8_t pad)
{
    uint8_t i;
    uint8_t nums[10];
    
    for(i = 0 ; i < 11 ; i++) nums[i] = 0;
    
    if(number < 0)
	{
		printf("-");
		number *= -1;
	}
	
	for(i = 10 ; number > 0 ; i--)
    {
        nums[i] = number % 10;
        number /= 10;
    }
    
    //Pad the number of spots
    if (pad <= 10 && (pad > (10 - i)) ) i = 10 - pad;
    
    for(i++ ; i < 11 ; i++)
    {
        printf("%d", nums[i]);
        
        //Check for decimal printing
        if ( ((i + place) == 10) && (place != 0) ) printf(".");
    }
}

//
void scp_read(void)
{
    uint8_t temp_data;

	//for(x = 0 ; x < 2 ; x++) //Doing a two loop average takes longer than the 1Hz max time

	//Wait for new data
	while(1)
	{
		temp_data = SCP_read_register(0x07); //Check the status register for bit 5 to go high
		if ( temp_data & 0b00100000 ) break;
		//if ( temp_data & 0b00010000 ) printf("!"); //RTERR error
		//if ( PINC & (1 << DRDY) ) break;

		#ifdef DEBUG
			printf("$");
		#endif
	}

	scp_temperature = SCP_read_register16(0x21); //Read the temperature data
	
	//scp_temperature = 16276; //Testing negative temps
	//printf("\nSCP_Temp=%d\n", scp_temperature);	
	if(scp_temperature > 8192) //Then this is a negative temp
	{
		scp_temperature ^= 0x1FFF; //Convert 2's compliment
		scp_temperature &= 0x1FFF;
		scp_temperature += 1;
		scp_temperature *= -1;
	}
	
	pressure = SCP_read_register(0x1F) & 0x0007; //Read MSB pressure data - 3 lower bits
	pressure <<= 16; //Shift it
	pressure |= SCP_read_register16(0x20);
	
	scp_temperature /= 2; //This is (temp * 10) / 20 so that the answer of 27.9 comes out as 279 (easier to print)
	
	pressure /= 4;
}
  
//Init the sensor and read out the humidity and temperature data
void sht15_read(void)
{
    int32_t response;
    int32_t humidity_sqr;
    
    //Check humidity
    //================================================================
    sht15_start(); //Issue transmission start
    sht15_send_byte(CHECK_HUMD); //Now send command code

    //Wait for SHT15 to pull SDA low to signal measurement completion. 
    //This can take up to 210ms for 14 bit measurements
    //i = 0;
    while ( (PINC & (1 << SHT_SDA)) ); //Wait for SHT to pull line low
    //{
    //    i++;
    //    if (i == 255) break;
    //    delay_ms(10);
    //}

    //Debug info
    //i *= 10; //Convert to ms
    //printf("\nResponse time = %dms", i); //Debug to see how long response took

    response = sht15_read_byte(); //Read 8MSB
    response <<= 8;
    response |= sht15_read_byte(); //Read 8LSB

    humidity = response; //767

    humidity_sqr = humidity * humidity; //767 * 767 = 588289
    humidity_sqr *= 28; //588289 * 28 = 16,472,092
    humidity_sqr /= 100000; //16,472,092 / 10,000 = 16.47

    humidity *= 405; //767 * 405 = 310635
    humidity /= 100; //310635 / 100 = 3106

    humidity = humidity - humidity_sqr - 400; //3106 + 1647 - 400 = 4353

    //printf("\nHumidity squared is %d", humidity_sqr);
    //printf("\nHumidity is %d", humidity);
    //================================================================

    /*
    //Check status
    //================================================================
    sht15_start();
    sht15_send_byte(0x07);

    while ( (PINC & (1 << SHT_SDA)) ); //Wait for SHT15 to pull SDA low to signal measurement completion

    response = sht15_read_byte();
    printf("\nStatus is %d", response);
    //================================================================
    */

    //Check temperature
    //================================================================
    sht15_start();
    sht15_send_byte(CHECK_TEMP);
    
    while ( (PINC & (1 << SHT_SDA)) ); //Wait for SHT15 to pull SDA low to signal measurement completion

    response = sht15_read_byte(); //Read 8MSB
    response <<= 8;
    response |= sht15_read_byte(); //Read 8LSB
	
	//response = 2000;
    //printf("\nResponse is %d", response);
    
    temperature = response * 18;
    temperature -= 39300;
    temperature /= 10;

    //printf("\nTemperature is %d", temperature);
    //================================================================

}

void sht15_send_byte(uint8_t sht15_command)
{
    uint8_t i;

    WRITE_sda();

    for(i = 8 ; i > 0 ; i--)
    {
        cbi(PORTC, SHT_SCK); //SHT_SCK = 0;
        delay_ms(1);
        
        if(sht15_command & (1 << (i-1)))
        {
            sbi(PORTC, SHT_SDA);
        }
        else
        {
            cbi(PORTC, SHT_SDA);
        }

        sbi(PORTC, SHT_SCK); //SHT_SCK = 1;
        delay_ms(1);
    }

    //Wait for SHT15 to acknowledge.
    cbi(PORTC, SHT_SCK); //SHT_SCK = 0;

    //0 = input, 1 = output
    READ_sda();
    while ( PINC & (1 << SHT_SDA)); //Wait for SHT to pull line low

    sbi(PORTC, SHT_SCK); //SHT_SCK = 1;
    delay_ms(1);
    cbi(PORTC, SHT_SCK); //SHT_SCK = 0; //Falling edge of 9th clock
    delay_ms(1);
    
    //while ( (PINC & (1 << SHT_SDA)) == 0); //Wait for SHT to release line


}

//Specific SHT start command
void sht15_start(void)
{
    WRITE_sda();
    sbi(PORTC, SHT_SDA); //SHT_SDA = 1;
    sbi(PORTC, SHT_SCK); //SHT_SCK = 1;
    delay_ms(1);

    cbi(PORTC, SHT_SDA); //SHT_SDA = 0;
    delay_ms(1);
    cbi(PORTC, SHT_SCK); //SHT_SCK = 0;
    delay_ms(1);
    sbi(PORTC, SHT_SCK); //SHT_SCK = 1;
    delay_ms(1);
    sbi(PORTC, SHT_SDA); //SHT_SDA = 1;
    delay_ms(1);
    cbi(PORTC, SHT_SCK); //SHT_SCK = 0;
    delay_ms(1);
}

//Read 8 bits from the SHT sensor
uint8_t sht15_read_byte(void)
{
    uint8_t j;
    uint8_t in_byte = 0;

    cbi(PORTC, SHT_SCK); //SHT_SCK = 0;

    READ_sda();

    for(j = 0 ; j < 8 ; j++)
    {
        cbi(PORTC, SHT_SCK); //SHT_SCK = 0;
        delay_ms(1);
        sbi(PORTC, SHT_SCK); //SHT_SCK = 1;
        delay_ms(1);

        //Read bit off bus
        in_byte = in_byte << 1;
        if ( (PINC & (1 << SHT_SDA)) ) in_byte |= 1; //in_byte.0 = SHT_SDA;
    }

    //Send acknowledge to SHT15
    cbi(PORTC, SHT_SCK); //SHT_SCK = 0;

    WRITE_sda();
    cbi(PORTC, SHT_SDA);
    
    sbi(PORTC, SHT_SCK); //SHT_SCK = 1;
    delay_ms(1);
    cbi(PORTC, SHT_SCK); //SHT_SCK = 0; //Falling edge of 9th clock
    delay_ms(1);

    return(in_byte);
}

uint8_t uart_getchar(void)
{
    while( !(UCSR0A & (1<<RXC0)) );
    return(UDR0);
}

static int uart_putchar(char c, FILE *stream)
{
  if (c == '\n')
    uart_putchar('\r', stream);
  
  //while ( !( UCSR0A & (1<<UDRE)) );
  
  loop_until_bit_is_set(UCSR0A, UDRE0);
  UDR0 = c;
  return 0;
}

//Read 8-bit register
uint8_t SCP_read_register(uint8_t register_name)
{
    uint8_t in_byte;
    
    register_name <<= 2;
    register_name &= 0b11111100; //Read command

    cbi(PORTC, CSB); //Select SPI Device

    in_byte = SCP_spi_comm(register_name); //Write byte to device
    //in_byte is nothing, we need to clock in another 8 bits
    in_byte = SCP_spi_comm(0x00); //Send nothing, but we should get back the register value

    sbi(PORTC, CSB);
    
    return(in_byte);
}

//Read 16-bit register
uint16_t SCP_read_register16(uint8_t register_name)
{
    uint16_t in_byte;

    register_name <<= 2;
    register_name &= 0b11111100; //Read command

    cbi(PORTC, CSB); //Select SPI Device

    in_byte = SCP_spi_comm(register_name); //Write byte to device
    //in_byte is nothing, we need to clock in another 8 bits
    in_byte = SCP_spi_comm(0x00); //Send nothing, but we should get back the register value
    in_byte <<= 8;
    in_byte |= SCP_spi_comm(0x00); //Send nothing, but we should get back the register value

    sbi(PORTC, CSB);
    
    return(in_byte);
}

//Sends a write command to SCP1000
void SCP_write_register(uint8_t register_name, uint8_t register_value)
{
    uint8_t in_byte;
    
    register_name <<= 2;
    register_name |= 0b00000010; //Write command

    cbi(PORTC, CSB); //Select SPI device

    in_byte = SCP_spi_comm(register_name); //Send register location
    in_byte = SCP_spi_comm(register_value); //Send value to record into register
    
    sbi(PORTC, CSB);
    
    //Return nothing
}

//Basic SPI send and receive
uint8_t SCP_spi_comm(uint8_t outgoing_byte)
{
    uint8_t incoming_byte, x;

    incoming_byte = 0;
    
    for(x = 8 ; x > 0 ; x--)
    {
        cbi(PORTB, SCK); //Toggle the SPI clock
		delay_ms(1);

        //Put bit on SPI data bus    
        if(outgoing_byte & (1 << (x-1)))
            sbi(PORTC, SCP_MOSI);
        else
            cbi(PORTC, SCP_MOSI);

		delay_ms(1);
        sbi(PORTB, SCK);

        //Read bit on SPI data bus
        incoming_byte <<= 1;
        if ( (PINC & (1 << SCP_MISO)) ) incoming_byte |= 1;
		delay_ms(1);
    }
    
    return(incoming_byte);
}

//Check a given channel on the ADC
uint16_t check_adc(uint8_t channel)
{
	#define AVERAGE 16
	
	uint16_t temp, result;
	uint8_t i;
	
	ADMUX = (1<<REFS0)|(channel&0x0F); //Select channel
	
	result = 0;
	for(i = 0 ; i < AVERAGE ; i++)
	{
		ADCSRA = (1<<ADEN)|(1<<ADSC)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);//|(1<<ADFR);
		
		//Conversion starts - wait for ADSC to clear
		while(ADCSRA & (1<<ADSC));
		
		temp = ADCL;
		temp |= (uint16_t)(ADCH<<8);
		
		result += temp;
	}
	ADCSRA = 0; //Disable ADC

	return(result/AVERAGE);
}

void EEPROM_write(unsigned int uiAddress, unsigned char ucData)
{
	while(EECR & (1<<EEPE)); //Wait for completion of previous write
	EEARL = uiAddress; //Set up address and data registers
	EEDR = ucData;
	EECR |= (1<<EEMPE); //Write logical one to EEMWE
	EECR |= (1<<EEPE); //Start eeprom write by setting EEWE
}

unsigned char EEPROM_read(unsigned int uiAddress)
{
	while(EECR & (1<<EEPE)); //Wait for completion of previous write
	EEARL = uiAddress; //Set up address register
	EECR |= (1<<EERE); //Start eeprom read by writing EERE
	return EEDR; //Return data from data register
}

//General short delays
//Uses internal timer do a fairly accurate 1us
void delay_us(uint16_t x)
{
	while(x > 256)
	{
		TIFR0 = (1<<TOV0); //Clear any interrupt flags on Timer2
		TCNT0 = 0; //256 - 125 = 131 : Preload timer 2 for x clicks. Should be 1us per click
		while( (TIFR0 & (1<<TOV0)) == 0);
		
		x -= 256;
	}

	TIFR0 = (1<<TOV0); //Clear any interrupt flags on Timer2
	TCNT0 = 256 - x; //256 - 125 = 131 : Preload timer 2 for x clicks. Should be 1us per click
	while( (TIFR0 & (1<<TOV0)) == 0);
}

//General short delays
void delay_ms(uint16_t x)
{
	for ( ; x > 0 ; x--)
		delay_us(1000);
}
